/* Generated automatically by H5make_libsettings -- do not edit */



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright by The HDF Group.                                               *
 * Copyright by the Board of Trustees of the University of Illinois.         *
 * All rights reserved.                                                      *
 *                                                                           *
 * This file is part of HDF5.  The full HDF5 copyright notice, including     *
 * terms governing use, modification, and redistribution, is contained in    *
 * the COPYING file, which can be found at the root of the source code       *
 * distribution tree, or in https://www.hdfgroup.org/licenses.               *
 * If you do not have access to either file, you may request a copy from     *
 * help@hdfgroup.org.                                                        *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Created:		May 20, 2024
 *			xbrbbot@76da895c3a1d
 *
 * Purpose:		This machine-generated source code contains
 *			information about the library build configuration
 *
 * Modifications:
 *
 *	DO NOT MAKE MODIFICATIONS TO THIS FILE!
 *	It was generated by code in `H5make_libsettings.c'.
 *
 *-------------------------------------------------------------------------
 */

#include "H5private.h"
H5_GCC_DIAG_OFF("larger-than=")

H5_CLANG_DIAG_OFF("overlength-strings")

char H5libhdf5_settings[]=
	"        SUMMARY OF THE HDF5 CONFIGURATION\n"
	"        =================================\n"
	"\n"
	"General Information:\n"
	"-------------------\n"
	"                   HDF5 Version: 1.13.2\n"
	"                  Configured on: 2022-08-15\n"
	"                  Configured by: Ninja\n"
	"                    Host system: Linux-5.15.0-87-generic\n"
	"              Uname information: Linux\n"
	"                       Byte sex: little-endian\n"
	"             Installation point: /usr\n"
	"\n"
	"Compiling Options:\n"
	"------------------\n"
	"                     Build Mode: RelWithDebInfo\n"
	"              Debugging Symbols: OFF\n"
	"                        Asserts: OFF\n"
	"                      Profiling: OFF\n"
	"             Optimization Level: OFF\n"
	"\n"
	"Linking Options:\n"
	"----------------\n"
	"                      Libraries: \n"
	"  Statically Linked Executables: OFF\n"
	"                        LDFLAGS: -Wl,-O1 -Wl,--hash-style=gnu -Wl,--as-needed -fmacro-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/hdf5-1.13.2=/usr/src/debug/hdf5/1.13.2-r0  -fdebug-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/hdf5-1.13.2=/usr/src/debug/hdf5/1.13.2-r0  -fmacro-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/build=/usr/src/debug/hdf5/1.13.2-r0  -fdebug-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/build=/usr/src/debug/hdf5/1.13.2-r0  -fdebug-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/recipe-sysroot=  -fmacro-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/recipe-sysroot=  -fdebug-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/recipe-sysroot-native=  -Wl,-z,relro,-z,now\n"
	"                     H5_LDFLAGS: \n"
	"                     AM_LDFLAGS: \n"
	"                Extra libraries: m;dl\n"
	"                       Archiver:\n"
	"                         Ranlib:\n"
	"\n"
	"Languages:\n"
	"----------\n"
	"                              C: YES\n"
	"                     C Compiler:\n"
	"                       CPPFLAGS:\n"
	"                    H5_CPPFLAGS: \n"
	"                    AM_CPPFLAGS: \n"
	"                         CFLAGS:   -std=c99  -mcpu=cortex-a72.cortex-a53 -march=armv8-a+crc -fstack-protector-strong  -O2 -D_FORTIFY_SOURCE=2 -Wformat -Wformat-security -Werror=format-security  --sysroot=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/recipe-sysroot  -O2 -pipe -g -feliminate-unused-debug-types -fmacro-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/hdf5-1.13.2=/usr/src/debug/hdf5/1.13.2-r0  -fdebug-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/hdf5-1.13.2=/usr/src/debug/hdf5/1.13.2-r0  -fmacro-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/build=/usr/src/debug/hdf5/1.13.2-r0  -fdebug-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/build=/usr/src/debug/hdf5/1.13.2-r0  -fdebug-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/recipe-sysroot=  -fmacro-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/recipe-sysroot=  -fdebug-prefix-map=/scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VersalNetFull/build/tmp/work/cortexa72-cortexa53-xilinx-linux/hdf5/1.13.2-r0/recipe-sysroot-native=  -fstdarg-opt -fdiagnostics-urls=never -fno-diagnostics-color -fmessage-length=0\n"
	"                      H5_CFLAGS: -Wall;-Wcast-qual;-Wconversion;-Wextra;-Wfloat-equal;-Wformat=2;-Winit-self;-Winvalid-pch;-Wmissing-include-dirs;-Wshadow;-Wundef;-Wwrite-strings;-pedantic;-Wno-c++-compat;-Wbad-function-cast;-Wcast-align;-Wformat;-Wimplicit-function-declaration;-Wint-to-pointer-cast;-Wmissing-declarations;-Wmissing-prototypes;-Wnested-externs;-Wold-style-definition;-Wpacked;-Wpointer-sign;-Wpointer-to-int-cast;-Wredundant-decls;-Wstrict-prototypes;-Wswitch;-Wunused-but-set-variable;-Wunused-variable;-Wunused-function;-Wunused-parameter;-finline-functions;-Wno-aggregate-return;-Wno-inline;-Wno-missing-format-attribute;-Wno-missing-noreturn;-Wno-overlength-strings;-Wlarger-than=2560;-Wlogical-op;-Wframe-larger-than=16384;-Wpacked-bitfield-compat;-Wsync-nand;-Wno-unsuffixed-float-constants;-Wdouble-promotion;-Wtrampolines;-Wstack-usage=8192;-Wmaybe-uninitialized;-Wno-jump-misses-init;-Wstrict-overflow=2;-Wno-suggest-attribute=const;-Wno-suggest-attribute=noreturn;-Wno-suggest-attribute=pure;-Wno-suggest-attribute=format;-Wdate-time;-Warray-bounds=2;-Wc99-c11-compat;-Wincompatible-pointer-types;-Wint-conversion;-Wshadow;-Wduplicated-cond;-Whsa;-Wnormalized;-Wnull-dereference;-Wunused-const-variable;-Walloca;-Walloc-zero;-Wduplicated-branches;-Wformat-overflow=2;-Wformat-truncation=1;-Wrestrict;-Wattribute-alias;-Wcast-align=strict;-Wshift-overflow=2;-Wcast-function-type;-Wmaybe-uninitialized;-Wno-suggest-attribute=cold;-Wno-suggest-attribute=malloc;-Wattribute-alias=2;-Wmissing-profile;-Wc11-c2x-compat\n"
	"                      AM_CFLAGS: \n"
	"               Shared C Library: YES\n"
	"               Static C Library: YES\n"
	"\n"
	"                        Fortran: OFF\n"
	"               Fortran Compiler:  \n"
	"                  Fortran Flags: \n"
	"               H5 Fortran Flags: \n"
	"               AM Fortran Flags: \n"
	"         Shared Fortran Library: YES\n"
	"         Static Fortran Library: YES\n"
	"\n"
	"                            C++: OFF\n"
	"                   C++ Compiler:\n"
	"                      C++ Flags:\n"
	"                   H5 C++ Flags: \n"
	"                   AM C++ Flags: \n"
	"             Shared C++ Library: YES\n"
	"             Static C++ Library: YES\n"
	"\n"
	"                            JAVA: OFF\n"
	"                 JAVA Compiler:  \n"
	"\n"
	"Features:\n"
	"---------\n"
	"                     Parallel HDF5: OFF\n"
	"  Parallel Filtered Dataset Writes: \n"
	"                Large Parallel I/O: \n"
	"                High-level library: ON\n"
	"Dimension scales w/ new references: \n"
	"                  Build HDF5 Tests: ON\n"
	"                  Build HDF5 Tools: ON\n"
	"       Build High-level HDF5 Tools: ON\n"
	"                      Threadsafety: OFF\n"
	"               Default API mapping: v114\n"
	"    With deprecated public symbols: ON\n"
	"            I/O filters (external): \n"
	"                               MPE: \n"
	"                        Direct VFD: \n"
	"                        Mirror VFD: \n"
	"                     Subfiling VFD: \n"
	"                (Read-Only) S3 VFD: \n"
	"              (Read-Only) HDFS VFD: \n"
	"                           dmalloc: \n"
	"    Packages w/ extra debug output: \n"
	"                       API Tracing: OFF\n"
	"              Using memory checker: OFF\n"
	"   Memory allocation sanity checks: OFF\n"
	"            Function Stack Tracing: OFF\n"
	"                  Use file locking: best-effort\n"
	"         Strict File Format Checks: OFF\n"
	"      Optimization Instrumentation: \n"
;

H5_GCC_DIAG_ON("larger-than=")
H5_CLANG_DIAG_OFF("overlength-strings")
